'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _getOwnPropertyNames = require('babel-runtime/core-js/object/get-own-property-names');

var _getOwnPropertyNames2 = _interopRequireDefault(_getOwnPropertyNames);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

exports.init = init;

var _eventemitter = require('eventemitter2');

var _eventemitter2 = _interopRequireDefault(_eventemitter);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _textTable = require('text-table');

var _textTable2 = _interopRequireDefault(_textTable);

var _wifiLocation = require('wifi-location');

var _wifiLocation2 = _interopRequireDefault(_wifiLocation);

var _stripAnsi = require('strip-ansi');

var _stripAnsi2 = _interopRequireDefault(_stripAnsi);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var pkg = require('../package');
var geo = require('node-geocoder')('google', 'http');
var uber = require('uber-api')(pkg.uber);

var Surger = function () {
  function Surger() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? (0, _create2.default)(null) : arguments[0];
    (0, _classCallCheck3.default)(this, Surger);

    this.options = options;
    this.estimates = [];
    this.emitter = new _eventemitter2.default({ wildcard: true });
    this.options.debug && this.emitter.on('debug.*', function (title) {
      var _console;

      for (var _len = arguments.length, message = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        message[_key - 1] = arguments[_key];
      }

      (_console = console).warn.apply(_console, [_chalk2.default.red.bold(this.event), _chalk2.default.green.bold(title)].concat(message));
    });
    this.getCoords(this.handleCoords.bind(this));
  }

  (0, _createClass3.default)(Surger, [{
    key: 'isNonEmptyObject',
    value: function isNonEmptyObject(obj) {
      return (0, _getOwnPropertyNames2.default)(obj || (0, _create2.default)(null)).length !== 0;
    }
  }, {
    key: 'flatten',
    value: function flatten(arr) {
      return arr.reduce(function (i, chunk) {
        return i.concat(chunk);
      });
    }
  }, {
    key: 'handleCoords',
    value: function handleCoords(err, _ref) {
      var lat = _ref.lat;
      var lng = _ref.lng;

      if (err) throw err;
      this.emitter.emit('debug.handleCoords', 'location confirmed from `getCoords`');
      this.getDetailsFromLatLng(this.handleDetails.bind(this), lat, lng);
      this.getPriceEstimateFromLatLng(this.handlePriceEstimate.bind(this), lat, lng);
    }
  }, {
    key: 'handlePriceEstimate',
    value: function handlePriceEstimate(err, estimates) {
      if (err) throw err;
      this.parseEstimates(estimates);
    }
  }, {
    key: 'handleDetails',
    value: function handleDetails(err, details) {
      if (err) throw err;
      this.parseDetails(this.flatten(details));
    }
  }, {
    key: 'parseEstimates',
    value: function parseEstimates(estimates) {
      console.log((0, _textTable2.default)(estimates));
    }
  }, {
    key: 'parseDetails',
    value: function parseDetails(details) {
      console.log('');
      console.log('Location:  Nearby', _chalk2.default.dim.blue(details.streetName));
      console.log('');
    }
  }, {
    key: 'getDetailsFromLatLng',
    value: function getDetailsFromLatLng(cb, lat, lng) {
      geo.reverse({ lat: lat, lon: lng }, function (err, res) {
        if (err) return cb(err);
        cb(null, res);
      });
    }
  }, {
    key: 'getPriceEstimateFromLatLng',
    value: function getPriceEstimateFromLatLng(cb, lat, lng) {
      var _this = this;

      var yay = _chalk2.default.dim.bgGreen.bold;
      var nay = _chalk2.default.white.bgRed.bold;
      this.emitter.emit('debug.getPriceEstimateFromLatLng', 'getting estimate');
      uber.getPriceEstimate({ sLat: lat, sLng: lng, eLat: lat, eLng: lng }, function (err, resp) {
        if (err) return cb(err);
        _this.estimates = resp.prices.map(function (type) {
          var good = yay(' ✔ GOOD ');
          var bad = nay(' $ SURGE ' + type.surge_multiplier + 'x [min. ' + type.estimate + ']');
          return [type.localized_display_name, // UberX, UberT, etc.
          type.surge_multiplier === 1 ? good : bad // Surge Pricing?
          ];
        });
        _this.emitter.emit('debug.estimates', 'estimates', (0, _stringify2.default)(_this.estimates.map(function (a) {
          return a.map(_stripAnsi2.default);
        })));
        cb(null, _this.estimates);
      });
    }
  }, {
    key: 'getCoords',
    value: function getCoords(cb) {
      var _this2 = this;

      this.emitter.emit('debug.getCoords', 'finding towers');
      _wifiLocation2.default.getTowers(function (err, towers) {
        _this2.emitter.emit('debug.getCoords', 'found ' + towers.length + ' tower(s)');
        _this2.emitter.emit('debug.getTowers', 'err', err);
        _this2.emitter.emit('debug.getTowers', 'towers', (0, _stringify2.default)(towers));
        if (err) return cb(err);
        _this2.emitter.emit('debug.getCoords', 'finding location');
        _wifiLocation2.default.getLocation(towers, function (err, location) {
          _this2.emitter.emit('debug.getLocation', 'err', err);
          _this2.emitter.emit('debug.getLocation', 'location', (0, _stringify2.default)(location));
          if (err) return cb(err);
          _this2.emitter.emit('debug.getCoords', 'confirming location');
          if (!_this2.isNonEmptyObject(location)) return cb(Error(_chalk2.default.dim.red('Briefly unable to locate you. Try again.')));
          var latitude = location.latitude;
          var longitude = location.longitude;

          var coords = { lat: latitude, lng: longitude };
          cb(null, coords);
        });
      });
    }
  }]);
  return Surger;
}();

function init(options) {
  return new Surger(options);
}